'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sdk = undefined;

var _tools = require('./utils/tools');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var doc = document;

// 用 Promise 包裹 sdk
var sdk = exports.sdk = new Promise(function (resolve, reject) {
  resolve();
});

// 不在 app 中，使用这里定义的处理
var appSdk = {
  debug: process.env.NODE_ENV === 'development', // 是否启用调试,再调试模式下参数检查情况会输出到控制台

  /**
   * 设置
   * @param debug
   */
  setDebug: function setDebug(debug) {
    this.debug = debug;
  },


  /*eslint-disable max-len*/
  /**
   * 获取platform协议的实现情况，比如可用的事件、可用的接口、协议版本等
   * @param callback
   * 返回的结果
   {
      code: 0,
      data: '*appRequest*authOutOfDate*buryEvent*buryPageEnd*buryPageStart*checkLoginStatus*checkNetwork*close*confirm*getAppCommonInfo*getCache*getCommonInfo*getPayLimit*onEvent*putCache*removeCache*request*setMenu*setTitle*setTitleBarVisible*showResultCtrlDialog*showTradePasswordInput*toast',
      time: '1505728066354'
    }
   */
  env: function env(callback) {
    // 1
    return sdk.then(function () {
      console.info('获取platform协议的实现情况,可用的事件、可用的接口、协议版本');
      /*eslint-disable max-len*/
      callback({
        code: 0,
        data: '*appRequest*authOutOfDate*buryEvent*buryPageEnd*buryPageStart*checkLoginStatus*checkNetwork*close*confirm*getAppCommonInfo*getCache*getCommonInfo*getPayLimit*onEvent*putCache*removeCache*request*setMenu*setTitle*setTitleBarVisible*showResultCtrlDialog*showTradePasswordInput*toast',
        time: '1505728066354'
      });
    });
  },


  /**
   * 设置 title
   * @param titleName
   */
  setTitle: function setTitle(param) {
    // 2
    if ((0, _tools.required)(this.debug, {
      param: param
    }, 'setTitle')) {

      return sdk.then(function () {
        console.info('设置浏览器标题');
        document.title = param.titleName;
        var i = document.createElement('iframe');
        i.src = '/favicon.ico';
        i.style.display = 'none';
        i.onload = function () {
          setTimeout(function () {
            i.remove();
          }, 9);
        };
        document.body.appendChild(i);
      });
    }
  },


  /**
   * 设置标题栏的显示/隐藏
   * @param visible {boolean}
   */
  setTitleBarVisible: function setTitleBarVisible() {
    var visible = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    // 3
    return sdk.then(function () {
      console.info('设置标题栏的显示/隐藏');
    });
  },


  /**
   * 设置当前页面头部右侧菜单
   * 单个直接显示在右侧，多个的话以下拉的形式显示
   * 目前实现：Android 当路由切换后会自动去掉右侧菜单
   * 但 IOS 需要手动清除掉右侧菜单，传入空数组即可
   * @param menus {Array}
   * 参数格式
   [{
      menuTitle: "扫一扫",
      menuImage: "http://localhost:8080/sao.png",
      menuAction: "function"
   }]
   *
   */
  setMenu: function setMenu() {
    var menus = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    // 4
    if (!Array.isArray(menus)) {
      if (this.debug) {
        console.log('setMenu, menus 必须是一个数组类型');
      }
      return;
    }
    for (var l = menus.length; l > 0; l--) {
      var _required;

      var _menu = menus[l - 1];
      if (!(0, _tools.required)(this.debug, (_required = {}, _defineProperty(_required, 'menus[' + l + '].menuTitle', _menu.menuTitle), _defineProperty(_required, 'menus[' + l + '].menuAction', _menu.menuAction), _required), 'setMenu')) {
        return;
      }
    }

    menus.forEach(function (it) {
      it.menuAction = (0, _tools.safeFunction)(it.menuAction);
    });

    return sdk.then(function () {
      if (!menus || menus.length === 0) {
        var menusEl = doc.getElementById('app-sdk-menu');
        doc.documentElement.removeChild(menusEl);
      } else {
        var menuEl = doc.createElement('div');
        menuEl.id = 'app-sdk-menu';
        menuEl.style.position = 'absolute';
        menuEl.style.top = '10px';
        menuEl.style.right = '10px';

        menus.forEach(function (it) {
          var subMenuEl = doc.createElement('a');
          subMenuEl.href = '#';
          subMenuEl.innerHTML = it.menuTitle;
          subMenuEl.addEventListener('click', function (e) {
            e.preventDefault();
            it.menuAction();
          }, false);
          menuEl.appendChild(subMenuEl);
        });
        doc.documentElement.appendChild(menuEl);
      }
      console.info('设置菜单');
    });
  },


  /**
   * 获取公共参数信息
   * @param callback {function}
   * 返回的数据格式为
   *
   {
    code: 0,
    time: 1505199918343,
    data: {
      appVersion: 'app 版本',
      channel: '渠道标识',
      clientName: '设备（ios，android',
      deviceId: '设备唯一标识',
      deviceType: '设备类型',
      mac: 'mac地址',
      osVersion: ''
    }
   }
   */
  getCommonInfo: function getCommonInfo(callback) {
    // 5
    if ((0, _tools.required)(this.debug, { callback: callback }, 'getCommonInfo')) {
      return sdk.then(function () {
        callback({
          code: 0,
          time: 1505199918343,
          data: {
            appVersion: 'app 版本',
            channel: '渠道标识',
            clientName: '设备（ios，android',
            deviceId: '设备唯一标识',
            deviceType: '设备类型',
            mac: 'mac地址',
            osVersion: ''
          }
        });
      });
    }
  },


  /**
   * 获取当前用户的基本信息
   * @param callback {function}
   * 返回的数据格式为
   *
   {
    code: 0,
    time: 1505199918343,
    data: {
      channelUserId: '渠道用户标识',
      hasOpenSecAccount: '是否已开通二类户'
    }
   }
   */
  getUserInfo: function getUserInfo(callback) {
    // 6
    if ((0, _tools.required)(this.debug, { callback: callback }, 'getUserInfo')) {
      return sdk.then(function () {
        callback({
          code: 0,
          time: 1505199918343,
          data: {
            channelUserId: '渠道用户标识',
            hasOpenSecAccount: '是否已开通二类户'
          }
        });
      });
    }
  },


  /**
   * 网络请求
   * @param param {{}}
   * param 格式
   {
    url: '请求url地址',
    httpMethod: 'GET 或 POST DELETE 等', // 注意是大写
    parameters: 'json 字符串' // JSON.stringify(json)
   }
   * @param callback {function}
   */
  request: function request(param, callback) {
    // 7
    if ((0, _tools.required)(this.debug, {
      param: param,
      'param.url': param && param.url,
      'param.httpMethod': param && param.httpMethod,
      'param.parameters': param && param.parameters,
      callback: callback
    }, 'request')) {

      return sdk.then(function () {
        console.info('网络请求');
      });
    }
  },


  /**
   * 弹出toast
   * @param param {{}} { msg(必填):String }
   */
  toast: function toast(param) {
    // 8
    if ((0, _tools.required)(this.debug, { param: param, 'param.msg': param && param.msg }, 'toast')) {
      return sdk.then(function () {
        console.info('弹出toast');
        // 这里可以走 h5 toast
      });
    }
  },


  /**
   * 确认提示框
   * @param param {{}} { title(必填):String, msg(必填):String, cancelBtnText:String, cancelCallBack:function, okBtnText:String, okCallBack:function }
   */
  confirm: function confirm(param) {
    // 9
    if ((0, _tools.required)(this.debug, {
      param: param,
      'param.title': param && param.title,
      'param.msg': param && param.msg
    }, 'confirm')) {

      return sdk.then(function () {
        console.info('弹出简单对话框');
      });
    }
  },


  /**
   * 关闭浏览器
   */
  close: function close() {
    // 10
    return sdk.then(function () {
      console.info('关闭浏览器');
    });
  },


  /**
   * 启动app模块，目前实现的模块有：登录：LOGIN（如果当前是登录态，则不会跳到登录页面，只会回调 callback）开户：OPENACCOUNT
   * @param param {{}}
   * @param callback {function}
   * 回调结果根据具体的模块返回不同的数据
   * name是LOGIN的情况下：返回值和 getUserInfo 返回值相同
   */
  start: function start(param, callback) {
    // 11
    if ((0, _tools.required)(this.debug, {
      param: param,
      'param.name': param && param.name,
      'param.param': param && param.param
    }, 'start')) {

      param.param = JSON.stringify(param.param);

      return sdk.then(function () {
        console.info('启动app模块');
      });
    }
  },


  /**
   * 检查设备的网络状况
   *
   * @param callback {function} 回调参数 : { hasNetwork:boolean, netType:String(wifi|2G|3G|4G|unknow) }
   */
  checkNetwork: function checkNetwork(callback) {
    // 12
    if ((0, _tools.required)(this.debug, { callback: callback }, 'checkNetwork')) {

      return sdk.then(function () {
        console.info('检查设备的网络状况');
        callback();
      });
    }
  },


  /**
   * 调用app的分享功能
   * @param param {{}} { type(必填): String[text|web|view], channel(必填): String[WX|QQ|WB|MON|STOCK] }
   * * 参数格式
   {
      channel: 'WX|WB|QQ|MON|STOCK', // 分享渠道， WX|WB|QQ|MON|STOCK 中间用竖线隔开，可选其中某个或某几个， 空串的话默认为全部。WX代表微信好友和微信朋友圈
      url: '', // Web分享地址
      title: '', //  Web分享标题
      desc: '', //  Web分享描述
      text: '', //  文本分享内容
      iconUrl: '', //  Web分享图标
      startY: '', //  截图起始 0-1的值 页面比例
      endY: '', //  截图尾部 0-1的值 页面比例
      type: '', // 分享类型（text, view, web）(类型为text时，text必填类型，为view时，startY和endY必填类型，为web时，url必填)
   }
    * @param callback {function}
   */
  share: function share(param, callback) {
    // 13
    if ((0, _tools.required)(this.debug, {
      param: param,
      'param.type': param && param.type,
      'param.channel': param && param.channel
    }, 'share')) {

      if (['text', 'web', 'view'].indexOf(param.type) === -1) {
        if (this.debug) {
          console.log('不正确的分享类型:', param.type);
        }
        return;
      }

      var channels = param.channel.split('|'),
          expectChannel = ['WX', 'QQ', 'WB', 'MON', 'STOCK'];
      for (var l = channels.length; l > 0; l--) {
        if (expectChannel.indexOf(channels[l - 1]) === -1) {
          if (this.debug) {
            console.log('不正确的分享渠道:', param.channel);
          }
          return;
        }
      }

      return sdk.then(function () {
        console.info('调用app的分享功能');
        callback();
      });
    }
  },


  /**
   * 进入页面埋点
   * @param param {page: 'page1'} 页面名称 (同一页面的页面名称应和 buryPageEnd 中的页面名称相同
   * @returns {Promise.<TResult>}
   */
  buryPageStart: function buryPageStart(param) {
    // 14
    if ((0, _tools.required)(this.debug, {
      param: param
    }, 'buryPageStart')) {

      return sdk.then(function () {
        console.info('');
      });
    }
  },


  /**
   * 离开页面埋点
   * @param param {page: 'page1'} 页面名称 (同一页面的页面名称应和 buryPageStart 中的页面名称相同
   * @returns {Promise.<TResult>}
   */
  buryPageEnd: function buryPageEnd(param) {
    // 15
    if ((0, _tools.required)(this.debug, {
      param: param
    }, 'buryPageEnd')) {

      return sdk.then(function () {
        console.info('');
      });
    }
  },


  /**
   * 埋点事件
   * @param param {}
   * 传参格式
   {
     eventID: '', //事件ID
     properties: {} // 事件属性
   }
   * @returns {Promise.<TResult>}
   */
  buryEvent: function buryEvent(param) {
    // 16
    if ((0, _tools.required)(this.debug, {
      param: param
    }, 'buryEvent')) {

      return sdk.then(function () {
        console.info('');
      });
    }
  },


  /**
   * 检查是否是登录状态
   * @param callback
   * 返回结果
   {
     code: 0,
     data: {
      isLogin: true // 是否为登录态
     },
     time: ''
   }
   * @returns {Promise.<TResult>}
   */
  checkLoginStatus: function checkLoginStatus(callback) {
    // 17
    return sdk.then(function () {
      console.info('检查是否是登录状态');
    });
  },


  /**
   * 放入本地缓存,考虑缓冲区已满的问题
   * @param {string} type local|session
   * @param key {String}
   * @param value {String}
   * @param callback {function}
   */
  putCache: function putCache(type, key, value, callback) {
    var _this = this;

    // 18
    if (!type || typeof key !== 'string') {
      return;
    }

    return sdk.then(function () {
      console.info('放入本地缓存,考虑缓冲区已满的问题');
      callback();
    }).catch(function (e) {
      if (_this.debug) {
        console.log(type, 'storage is full.');
      }
    });
  },


  /**
   * 从本地缓冲区获取数据,可以一次取多个key,以spliter分割,默认为';'
   * @param type {String} local|session
   * @param key {String}
   * @returns {{}} json
   * @param callback {function}
   */
  getCache: function getCache(type, key, callback) {
    // 19
    if (!type || typeof key !== 'string') {
      return;
    }
    return sdk.then(function () {
      console.info('从本地缓冲区获取数据,可以一次取多个key,以spliter分割,默认为\';\'');
      callback();
    });
  },


  /**
   * 从缓冲区移除数据,可以一次删除多个key,以spliter分割,默认为';'
   * @param type {String} local|session
   * @param key {String}
   * @param callback {function}
   */
  removeCache: function removeCache(type, key, callback) {
    // 20
    if (!type || typeof key !== 'string') {
      return;
    }
    return sdk.then(function () {
      console.info('从缓冲区移除数据,可以一次删除多个key,以spliter分割,默认为\';\'');
      callback();
    });
  },


  /************ 事件监听 **************/
  /**
   * app返回键监听
   * @param callback
   */
  onBackPress: function onBackPress(callback) {
    // 21
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onBackPress')) {
      return sdk.then(function () {
        console.info('app返回键监听');
        callback();
      });
    }
  },

  /**
   * webview关闭监听
   * @param callback
   */
  onClose: function onClose(callback) {
    // 22
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onClose')) {

      return sdk.then(function () {
        console.info('webview关闭监听');
        callback();
      });
    }
  },

  /**
   * 当前用户被登出监听
   * @param callback
   */
  onLogout: function onLogout(callback) {
    // 23
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onLogout')) {

      return sdk.then(function () {
        console.info('当前用户被登出监听');
        callback();
      });
    }
  },

  /**
   * 设备网络情况监听
   * @param callback
   */
  onNetChanged: function onNetChanged(callback) {
    // 24
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onNetChanged')) {
      return sdk.then(function () {
        console.info('设备网络情况监听');
        callback();
      });
    }
  },

  /**
   * webview切到后台监听
   * @param callback
   */
  onBackground: function onBackground(callback) {
    // 25
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onBackground')) {
      return sdk.then(function () {
        console.info('webview切到后台监听');
        callback();
      });
    }
  },

  /**
   * webview回到前台监听
   * 只有先触发onBackground才能触发onForeground
   * @param callback
   */
  onForeground: function onForeground(callback) {
    // 26
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onForeground')) {
      return sdk.then(function () {
        console.info('webview回到前台监听');
        callback();
      });
    }
  },

  /**
   * 设备摇一摇监听
   * @param callback
   */
  onShake: function onShake(callback) {
    // 27
    if ((0, _tools.required)(this.debug, {
      callback: callback
    }, 'onShake')) {
      return sdk.then(function () {
        console.info('设备摇一摇监听');
        callback();
      });
    }
  },

  /**
   * app推送消息监听
   * @param busisessType 指定监听的业务类型
   * @param callback
   */
  onPush: function onPush(busisessType, callback) {
    // 28
    if ((0, _tools.required)(this.debug, {
      busisessType: busisessType,
      callback: callback
    }, 'onPush')) {

      return sdk.then(function () {
        console.info('app推送消息监听');
        callback();
      });
    }
  },


  // 直销银行App额外方法

  /**
   * 获取 App 公共参数信息
   * @param callback {function}
   * 返回的数据格式为
   *
   {
    code: 0,
    time: 1505199918343,
    data: {
      appVersion: 'app 版本',
      auth: '753b72ef82faa1ca96a9137bfcf5dbd0',
      channel: '渠道标识',
      clientName: '设备（ios，android',
      custId: '101353543487838019584',
      deviceId: '设备唯一标识',
      deviceType: '设备类型',
      deviceName: '设备名称',
      mac: 'mac地址'
    }
   }
   */
  getAppCommonInfo: function getAppCommonInfo(callback) {
    // 29
    if ((0, _tools.required)(this.debug, { callback: callback }, 'getAppCommonInfo')) {
      return sdk.then(function () {
        callback();
      });
    }
  },


  /**
   * 自定义的非标准SaaS服务的网络请求
   * @param param {{}}
   * param 格式
   {
    url: '请求url地址',
    httpMethod: 'GET 或 POST DELETE 等', // 注意是大写
    parameters: 'json 字符串' // JSON.stringify(json)
   }
   * @param callback {function}
   */
  appRequest: function appRequest(param, callback) {
    // 30
    if ((0, _tools.required)(this.debug, {
      param: param,
      'param.url': param && param.url,
      'param.httpMethod': param && param.httpMethod,
      'param.parameters': param && param.parameters,
      callback: callback
    }, 'request')) {

      return sdk.then(function () {
        console.info('自定义的非标准SaaS服务的网络请求');
      });
    }
  },


  /**
   * 调用密码键盘
   * @param callback {function}
   * * 返回的数据格式为
   *
   {
    code: 0,
    time: 1505199918343,
    data: {
      timeString: '',
      password: '加密后的密码'
    }
   }
   */
  showTradePasswordInput: function showTradePasswordInput(callback) {
    // 31
    if ((0, _tools.required)(this.debug, { callback: callback }, 'showTradePasswordInput')) {
      return sdk.then(function () {
        console.info('调用密码键盘');
      });
    }
  },


  /**
   * 登录超时或者单点登录 auth 失效
   */
  authOutOfDate: function authOutOfDate() {
    // 32
    return sdk.then(function () {
      console.info('登录超时或者单点登录 auth 失效');
    });
  },


  /**
   * 弹出ResultControl对话框
   * @param callback {function}
   */
  showResultCtrlDialog: function showResultCtrlDialog(callback) {
    // 33
    if ((0, _tools.required)(this.debug, { callback: callback }, 'showResultCtrlDialog')) {
      return sdk.then(function () {
        console.info('弹出ResultControl对话框');
      });
    }
  }
};

exports.default = appSdk;