/**
 * 在调用invoke方法时，方法名加入后缀，用于直接映射native方法。
 * @param name
 * @returns {string}
 */
export function wrapperNativeName(name) {
  return `${name}:callback:`;
}

/**
 * 保证返回一个Function
 * @param callback
 * @returns {Function}
 */
export function safeFunction(callback) {
  return callback || function () {
  };
}

/**
 * 做参数的必填验证
 * @param debug
 * @param requireObj
 * @param fnName
 * @returns {boolean}
 */
export function required(debug = false, requireObj, fnName) {
  if (!requireObj) {
    return true;
  }
  const keys = Object.keys(requireObj);
  for (let i = 0; i < keys.length; i++) {
    if (!requireObj[keys[i]] && debug && fnName) {
      console.log(`${fnName}中参数 ${keys[i]} 必须填写!`);
      return false;
    }
  }

  return true;
}

export default {
  wrapperNativeName,
  safeFunction,
  required
};
